import sqlite3 as sq3

class Establish:
    def __init__(self, path, type="sqlite3"):
        self.type = type
        self.path = path
        self.connection = ""
        self.cursor = ""
        
        if(self.type == "sqlite3"):
            connection=sq3.connect(self.path)
            self.connection=sq3.connect(self.path)
            self.cursor=connection.cursor()            

    def getConnection(self):
        return self.connection, self.cursor


# Files
from . import file
from . import string
from . import json
from . import formadator
from . import time
from . import array
from . import attribute
from . import logger
from . import http
from . import database
from . import dicts
from . import token
from . import output
from . import system
from . import shredder
from . import url

# Folders
# from . import lookup

def files():
    print(16)


import re
import traceback

from .attribute import Text


def getPrevVariableName(var):
    stack = traceback.extract_stack()
    _,_,_, code = stack[-3]
    vars_name = re.compile(r'\((.*?)\).*$').search(code).groups()[0]
    return vars_name


def debug(*args):
    var_names=getPrevVariableName(args).replace(" ","").split(",")
    streamValue = ""
    for index,val in enumerate(args):
        streamValue += f'''{Text.Color.fg.cyan}{var_names[index]}{Text.Color.fg.endc}''' + ": " + str(val)
        if(index != len(args)-1):
            streamValue += "\n"

    print(streamValue)
    return streamValue


class STATUS:
    def __init__(self):

        self.CONTINUE = 100
        self.SWITCHING_PROTOCOLS = 101
        self.PROCESSING = 102
        
        self.OK = 200
        self.CREATED = 201
        self.ACCEPTED = 202
        self.NON_AUTHORITATIVE_INFORMATION = 203
        self.NO_CONTENT = 204	
        self.RESET_CONTENT = 205
        self.PARTIAL_CONTENT = 206
        self.MULTI_STATUS = 207	
        
        self.MULTIPLE_CHOICES = 300
        self.MOVED_PERMANENTLY = 301
        self.MOVED_TEMPORARILY = 302
        self.SEE_OTHER = 303	
        self.NOT_MODIFIED = 304	
        self.USE_PROXY = 305	
        self.TEMPORARY_REDIRECT = 307
        self.PERMANENT_REDIRECT = 308
        
        self.BAD_REQUEST = 400	
        self.UNAUTHORIZED = 401
        self.PAYMENT_REQUIRED = 402
        self.FORBIDDEN = 403	
        self.NOT_FOUND = 404	
        self.METHOD_NOT_ALLOWED = 405
        self.NOT_ACCEPTABLE = 406	
        self.PROXY_AUTHENTICATION_REQUIRED = 407
        self.REQUEST_TIMEOUT = 408	
        self.CONFLICT = 409	
        self.GONE = 410	
        self.LENGTH_REQUIRED = 411
        self.PRECONDITION_FAILED = 412
        self.REQUEST_TOO_LONG = 413	
        self.REQUEST_URI_TOO_LONG = 414	
        self.UNSUPPORTED_MEDIA_TYPE = 415	
        self.REQUESTED_RANGE_NOT_SATISFIABLE = 416
        self.EXPECTATION_FAILED = 417
        self.IM_A_TEAPOT = 418	
        self.INSUFFICIENT_SPACE_ON_RESOURCE = 419
        self.METHOD_FAILURE = 420	
        self.MISDIRECTED_REQUEST = 421
        self.UNPROCESSABLE_ENTITY = 422
        self.LOCKED = 423	
        self.FAILED_DEPENDENCY = 424	
        self.PRECONDITION_REQUIRED = 428
        self.TOO_MANY_REQUESTS = 429	
        self.REQUEST_HEADER_FIELDS_TOO_LARGE = 431
        self.UNAVAILABLE_FOR_LEGAL_REASONS = 451
        
        self.INTERNAL_SERVER_ERROR = 500
        self.NOT_IMPLEMENTED = 501
        self.BAD_GATEWAY = 502	
        self.SERVICE_UNAVAILABLE = 503
        self.GATEWAY_TIMEOUT = 504	
        self.HTTP_VERSION_NOT_SUPPORTED = 505
        self.INSUFFICIENT_STORAGE = 507	
        self.NETWORK_AUTHENTICATION_REQUIRED = 511


import re
import traceback
import json


class DictToObject:
    def __init__(self, **entries):
        self.dictionary=entries
        self.__dict__.update(entries)
    def __repr__(self):
        return repr(self.dictionary)


def getPrevVariableName(var):
    stack = traceback.extract_stack()
    # filename, lineno, function_name, code = stack[-3]
    _,_,_, code = stack[-3]
    vars_name = re.compile(r'\((.*?)\).*$').search(code).groups()[0]
    return vars_name


def jsObject(*args):
    dictObject={}
    var_names=getPrevVariableName(args).replace(" ","").split(",")
    for index,val in enumerate(args):
        dictObject[var_names[index]]=val
        # The below formula needs serious rectification
        # if(type(val)==type(dict())):
        #     dictObject[var_names[index]] = jsObject(val)
        # else:
        #     dictObject[var_names[index]]=val
    dictObject=DictToObject(**dictObject)
    return dictObject


def jsObjectToDict(jsObject):
    dictNew = {}
    for key, val in jsObject.dictionary.items():
        if(type(val)==type(DictToObject())):
            dictNew[key] = jsObjectToDict(val)
        else:
            dictNew[key] = val
    return dictNew


def jsDictToObject(dictionaryHere):
    dictNew = {}
    for key, val in dictionaryHere.items():
        if(type(val)==type(dict())):
            dictNew[key] = jsDictToObject(val)
        else:
            dictNew[key] = val
    return DictToObject(**dictNew)


def parseQueryStringParamsToJson(full_path):
    qs={}
    if("?" in full_path):
        query=full_path[full_path.index("?")+1:]
        query_tups=query.split("&")
        for qrt in query_tups:
            key,val=qrt.split("=")
            qs[key]=val
    return json.dumps(qs)


from . import file
from . import string

def searchFileContent(folder_path, search_string_list, isPrint=True, ignoring_extensions = []):
    def controlledPrint(input_stream, isPrint):
        if(isPrint==True):print(input_stream)

    tfft = file.FolderTraversal(folder_path)

    for search_string in search_string_list:        
        controlledPrint("<start> for: "+search_string, isPrint)
        find_dict={}
        temp_list=[]
            
        for itfft in tfft:
            if(string.stringMatchFromList(itfft, ignoring_extensions)==True):
                continue

            else:
                try:
                    if((".html") in itfft):
                        content = file.readFileHtml(itfft)

                    else:
                        content = file.readFile(itfft)

                    contentBig=content.upper()
                    contentSmall=content.lower()
                    contentShredded=contentSmall.replace(" ","").replace("\n","").replace("\t","")
                    search_string_small=search_string.lower()
                    
                    if(search_string in content):
                        input_stream=search_string + " : Organic exists in: " + itfft
                        controlledPrint(input_stream, isPrint)
                        temp_list.append((itfft,"Organic"))

                    if(search_string in contentBig):
                        input_stream= search_string + " : Big exists in: " + itfft
                        controlledPrint(input_stream, isPrint)
                        temp_list.append((itfft,"Big"))

                    if(search_string in contentSmall):
                        input_stream=search_string + " : Small exists in: " + itfft
                        controlledPrint(input_stream, isPrint)
                        temp_list.append((itfft,"Small"))

                    
                    if(search_string_small in contentSmall):
                        input_stream=search_string + " : Flattened exists in: " + itfft
                        controlledPrint(input_stream, isPrint)
                        temp_list.append((itfft,"Flattened"))

                    
                    if(search_string_small in contentShredded):
                        input_stream=search_string + " : Shredded exists in: " + itfft
                        controlledPrint(input_stream, isPrint)
                        temp_list.append((itfft,"Shredded"))

                except:
                    print("Soft Error: ", itfft," :was not read")

        if(len(temp_list)>0):
            find_dict[search_string]=temp_list


        controlledPrint("<end> for: " + search_string + "\n\n", isPrint)
    return find_dict


def vanishPyCache(folder_path):
    vanishing_extension=".pyc"

    tfft = file.FolderTraversal(folder_path)
    pycache_file_count=0
    
    for itfft in tfft:
        input_string=itfft
        if(vanishing_extension==input_string[-4:]):
            pycache_file_count+=1
            file.removeFile(input_string)

    print("Total: {} pycache files deleted from {}".fromat(pycache_file_count, folder_path))
    return None


def vanishPyCacheFolder(folder_path):
    vanishing_extension_file=".pyc"
    vanishing_extension_folder="__pycache__"

    tfft = file.FolderTraversal(folder_path)
    pycache_file_count=0
    
    for itfft in tfft:
        input_string=itfft
        if(vanishing_extension_folder in input_string):
            pycache_folder_path=input_string[:input_string.rindex("/")]
            pycache_file_count+=1

            file.deleteFolder(pycache_folder_path)

    print("Total: {} pycache folders deleted from {}".fromat(pycache_file_count, folder_path))
    return None


def splitStrDateTime(dash_datetime):
    year,month,day,hour,minute,second,msec=dash_datetime.split("_")
    return year,month,day,hour,minute,second,msec


def twentyFourToTwelveLower(hour, minute):
    ihour=int(hour)
    if(ihour>=12):
        ihour=ihour-12
        message=f'''{str(ihour)}:{minute} pm'''

    else:
        message=f'''{str(ihour)}:{minute} am'''

    return message


def twentyFourToTwelveUpper(hour, minute):
    ihour=int(hour)
    if(ihour>=12):
        ihour=ihour-12
        message=f'''{str(ihour)}:{minute} PM'''

    else:
        message=f'''{str(ihour)}:{minute} AM'''

    return message


def getStringDateTimeMessage(dash_datetime, number_to_month):
    year,month,day,hour,minute,second,msec=splitStrDateTime(dash_datetime)

    if(month[0]=="0"):month=month[1]
    month_stream=number_to_month[month]
    message=f'''{month_stream} {day}, {year} at {twentyFourToTwelveLower(hour, minute)}'''

    return message


import random
import datetime


def generateRandomStream(moreLen = 16, date = False, allowed = ['l','u','n','s']):
    chars = ""
    output = ""

    lowers = "abcdefghijklmnopqrstuvwxyz"
    uppers = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    numbers = "0123456789"
    specials = '''~`!@#$%^&*()_-+={[}]|\:;"'<,>.?/'''

    for a in allowed:
        a = a.lower()
        if(a == 'l'): chars += lowers
        if(a == 'u'): chars += uppers
        if(a == 'n'): chars += numbers
        if(a == 's'): chars += specials
    
    if(type(moreLen) == type(1)):
        if((moreLen > 0) == False):
            moreLen = 16
        else:
            pass
    else: moreLen = 16      
    
    for _ in range(moreLen):
        output += chars[random.randint(0,len(chars)-1)]
    
    if(date == True):
        ddns = str(datetime.datetime.now()).replace("-","").replace(" ","").replace(":","").replace(".","")
        output += ddns
    
    return output


import validators
# pip3 install validators

from .lookup import extension


def preprocess_url(input_url):
    output_url=input_url
    has_cleaned="0"
    
    if(output_url[-1]!="/"):
        output_url+="/"
        has_cleaned="1"

    if("?" in output_url):
        output_url=output_url[:output_url.index("?")]
        has_cleaned="1"

    if("//" in output_url):
        output_url=output_url.replace("//", "/")
        has_cleaned="1"

    while(output_url[-2:]=="//"):
        # output_url=output_url.replace("//","/")
        output_url=output_url[:-1]
        has_cleaned="1"

    return output_url,has_cleaned


class urlClass:
    def __init__(self, url=None, protocol=None, hostname=None, 
    pathname=None, url_valid=None, hostname_valid=None,
    url_extension=None, url_type=None):
        self.url=url
        self.protocol=protocol
        self.hostname=hostname
        self.pathname=pathname
        self.url_valid=url_valid
        self.hostname_valid=hostname_valid
        self.url_extension=url_extension
        self.url_type=url_type


def preprocessUrl(url):
    
    while(True):
        if(url[0]=="/"):
            url=url[1:]
            continue
        else:
            break

    if("?" in url):
        qindex=url.index("?")
        url=url[:qindex]
    
    if("#" in url):
        hindex=url.index("#")
        url=url[:hindex]

    if("://" not in url and "//" not in url):
        url="http://"+url

    return url


def validateUrl(url):
    
    valid=validators.url(url)
    
    if(valid==True):
        valid=True
    else:
        valid=False

    return valid


def validateHostName(hostname):
    
    hostname = hostname.lower()
    valid_chars = ".abcdefghijklmnopqrstuvwxyz0123456789"
    hostname_valid = True

    # this is a very loose hostname invalidation
    if(len(hostname)>63):
        hostname_valid=False
        return hostname_valid

    for hn in hostname:
        if(hn in valid_chars):
            continue
        else:
            hostname_valid=False
            return hostname_valid

    return hostname_valid


def getUrlExtension(pathname):
    
    if("." in pathname):
        dindex=pathname.index(".")
        extension=pathname[dindex+1:]
    else:
        extension="None"
    
    return extension


def objectifyUrl(url):
    
    # This objectification is not 100% definate
    # Will be 100% definate very sooon
    url=preprocessUrl(url)
    url_valid=validateUrl(url)
    
    stripped_url=url

    if(url[:7]=="http://"):
        protocol="http"
        stripped_url=stripped_url.replace(url[:7], "")
    elif(url[:8]=="https://"):
        protocol="https"
        stripped_url=stripped_url.replace(url[:8], "")
    elif("://" in url):
        lindex=url.index("://")
        protocol=url[:lindex]
        stripped_url=url[lindex+3:]
    elif("//" in url and "://" not in url):
        lindex=url.index("//")
        protocol=url[:lindex]
        stripped_url=url[lindex+2:]
    else:
        protocol="http"
        stripped_url=stripped_url.replace(url[:7], "")

    if("/" in stripped_url):
        sindex=stripped_url.index("/")
        hostname=stripped_url[:sindex]
        pathname=stripped_url[sindex+1:]
        if(pathname=="/"):
            pathname=""

    else:
        hostname=stripped_url
        pathname=""
    
    hostname_valid=validateHostName(hostname)
    url_extension=getUrlExtension(pathname)    

    if(url_extension in extension.file_extensions):
        url_type = extension.file_extensions[url_extension]
    else:
        url_type = "UNKNOWN"


    urlObject=urlClass(url, protocol, hostname, pathname, url_valid, hostname_valid, url_extension, url_type)
    return urlObject


file_extensions={
"None":"text",
"ogm":"video",
"doc":"text",
"class":"code",
"js":"code",
"swift":"code",
"cc":"code",
"tga":"image",
"ape":"audio",
"woff2":"font",
"cab":"archiv",
"whl":"archiv",
"mpe":"video",
"rmvb":"video",
"srt":"video",
"pdf":"text",
"xz":"archiv",
"exe":"exec",
"m4a":"audio",
"crx":"exec",
"vob":"video",
"tif":"image",
"gz":"archiv",
"roq":"video",
"m4v":"video",
"gif":"image",
"rb":"code",
"3g2":"video",
"m4":"code",
"ar":"archiv",
"vb":"code",
"sid":"audio",
"ai":"image",
"wma":"audio",
"pea":"archiv",
"bmp":"image",
"py":"code",
"mp4":"video",
"m4p":"video",
"ods":"sheet",
"jpeg":"image",
"command":"exec",
"azw4":"book",
"otf":"font",
"ebook":"text",
"rtf":"text",
"ttf":"font",
"mobi":"book",
"ra":"audio",
"flv":"video",
"ogv":"video",
"mpg":"video",
"xls":"sheet",
"jpg":"image",
"mkv":"video",
"nsv":"video",
"mp3":"audio",
"kmz":"image",
"java":"code",
"lua":"code",
"m2v":"video",
"deb":"archiv",
"rst":"text",
"csv":"sheet",
"pls":"audio",
"pak":"archiv",
"egg":"archiv",
"tlz":"archiv",
"c":"code",
"cbz":"book",
"xcodeproj":"code",
"iso":"archiv",
"xm":"audio",
"azw":"book",
"webm":"video",
"3ds":"image",
"azw6":"book",
"azw3":"book",
"php":"code",
"kml":"image",
"woff":"font",
"log":"text",
"zipx":"archiv",
"3gp":"video",
"po":"code",
"mpa":"audio",
"mng":"video",
"wps":"text",
"wpd":"text",
"a":"archiv",
"s7z":"archiv",
"ics":"sheet",
"tex":"text",
"go":"code",
"ps":"image",
"org":"text",
"sh":"exec",
"msg":"text",
"xml":"code",
"cpio":"archiv",
"epub":"book",
"docx":"text",
"lha":"archiv",
"flac":"audio",
"odp":"slide",
"wmv":"video",
"vcxproj":"code",
"mar":"archiv",
"eot":"font",
"less":"web",
"asf":"video",
"apk":"archiv",
"css":"web",
"mp2":"video",
"odt":"text",
"patch":"code",
"wav":"audio",
"msi":"exec",
"rs":"code",
"gsm":"audio",
"ogg":"video",
"cbr":"book",
"azw1":"book",
"m":"code",
"dds":"image",
"h":"code",
"dmg":"archiv",
"mid":"audio",
"psd":"image",
"dwg":"image",
"aac":"audio",
"s3m":"audio",
"cs":"code",
"cpp":"code",
"au":"audio",
"aiff":"audio",
"diff":"code",
"avi":"video",
"bat":"exec",
"jsp":"code",
"htm":"code",
"html":"code",
"pages":"text",
"bin":"exec",
"txt":"text",
"rpm":"archiv",
"m3u":"audio",
"max":"image",
"vcf":"sheet",
"svg":"image",
"ppt":"slide",
"clj":"code",
"png":"image",
"svi":"video",
"tiff":"image",
"tgz":"archiv",
"mxf":"video",
"7z":"archiv",
"drc":"video",
"yuv":"video",
"mov":"video",
"tbz2":"archiv",
"bz2":"archiv",
"gpx":"image",
"shar":"archiv",
"xcf":"image",
"dxf":"image",
"jar":"archiv",
"qt":"video",
"tar":"archiv",
"xpi":"archiv",
"zip":"archiv",
"thm":"image",
"cxx":"code",
"3dm":"image",
"rar":"archiv",
"md":"text",
"scss":"web",
"mpv":"video",
"webp":"image",
"war":"archiv",
"pl":"code",
"xlsx":"sheet",
"mpeg":"video",
"aaf":"video",
"avchd":"video",
"mod":"audio",
"rm":"video",
"it":"audio",
"wasm":"web",
"el":"code",
"eps":"image",
"OTHER":"OTHER"
}


video_extensions={
"ogm":"video",
"mpe":"video",
"rmvb":"video",
"srt":"video",
"vob":"video",
"roq":"video",
"m4v":"video",
"3g2":"video",
"mp4":"video",
"m4p":"video",
"flv":"video",
"ogv":"video",
"mpg":"video",
"mkv":"video",
"nsv":"video",
"m2v":"video",
"webm":"video",
"3gp":"video",
"mng":"video",
"wmv":"video",
"asf":"video",
"mp2":"video",
"ogg":"video",
"avi":"video",
"svi":"video",
"mxf":"video",
"drc":"video",
"yuv":"video",
"mov":"video",
"qt":"video",
"mpv":"video",
"mpeg":"video",
"aaf":"video",
"avchd":"video",
"rm":"video"
}



audio_extensions={
"ape":"audio",
"m4a":"audio",
"sid":"audio",
"wma":"audio",
"ra":"audio",
"mp3":"audio",
"pls":"audio",
"xm":"audio",
"mpa":"audio",
"flac":"audio",
"wav":"audio",
"gsm":"audio",
"mid":"audio",
"aac":"audio",
"s3m":"audio",
"au":"audio",
"aiff":"audio",
"m3u":"audio",
"mod":"audio",
"it":"audio"
}


image_extensions={
"tga":"image",
"tif":"image",
"gif":"image",
"ai":"image",
"bmp":"image",
"jpeg":"image",
"jpg":"image",
"kmz":"image",
"3ds":"image",
"kml":"image",
"ps":"image",
"dds":"image",
"psd":"image",
"dwg":"image",
"max":"image",
"svg":"image",
"png":"image",
"tiff":"image",
"gpx":"image",
"xcf":"image",
"dxf":"image",
"thm":"image",
"3dm":"image",
"webp":"image",
"eps":"image"    
}



from requests import Session
session = Session()

def makePostRequest(url, bodyObject, queryObject = None, headers = None):
    sp = session.post(url, data = bodyObject)
    return sp

def proxyRequest(baseUrl, proxyUrl, body, query, header):
    # baseUrl: original url
    # proxyUrl: proxy url
    
    # Under Construction
    return None


from . import system
import io
import os
from os import path

def removeFile(file_path):
    os.remove(file_path)
    return None

def readFile(file_name):
    with open(file_name, "r", encoding='latin-1') as f:
        text = f.read()
        f.close()
    return text   

def writeFile(file_name,content):
    with open(file_name,'a') as f:
        f.write(content)
        f.close()
    return None

def clearFile(file_name):
    with open(file_name,'w') as f:
        f.close()
    return None

def createFile(file_name):
    with open(file_name,'w') as f:#overrite   
        f.write("")
        f.close()
    return None  

def getSize(inputPath):
    return os.path.getsize(inputPath)

def createFolder(folder_path):
    os.mkdir(folder_path)
    
def deleteFolder(folderPath):
    system.shell('rm -rf {}'.format(folderPath))

def isPathExist(path_name):
    if(path.exists(path_name)):
        return True
    return False

def slashMatch(path,startslash,endslash):
    slashCount=1
    start_index=0
    end_index=len(path)

    for ip,p in enumerate(path):
        if(p=="/"):
            if(endslash!=-1):
                if(slashCount==startslash):
                    start_index=ip+1
                if(slashCount== endslash):
                    end_index=ip
                slashCount+=1
            else:
                if(slashCount==startslash):
                    start_index=ip+1
                slashCount+=1

    return path[start_index:end_index]

def slashMatchSlash(path, startslash,endslash):
    slashCount=1
    start_index=0
    end_index=len(path)

    for ip,p in enumerate(path):
        if(p=="/"):
            if(endslash!=-1):
                if(slashCount==startslash):
                    start_index=ip+1
                if(slashCount== endslash):
                    end_index=ip
                slashCount+=1
            else:
                if(slashCount==startslash):
                    start_index=ip+1
                slashCount+=1
    return path[start_index:end_index]+"/"

def createPath(path):
    path=path.replace('\\',"/")
    slash_count=path.count("/")

    for sc in range(slash_count):
        smp=slashMatchSlash(path, 0,sc+1)
        
        if(isPathExist(smp)==False and isPathExist(smp[:-1])==False):
            if(smp[-1]=="/"):os.mkdir(smp)

    if(isPathExist(path)==False):
        if(path[-1]!="/"):createFile(path)

    return None

def extractFileName(path):
    ind=-1
    while(True):
        p=path[ind]
        if(p=="/"):
            break
        ind-=1

    return path[ind+1:]


def listDirPath(input_path):
    return [input_path+f for f in os.listdir(input_path)]

def listDirPathSlash(input_path):
    return [input_path+f+"/" for f in os.listdir(input_path)]

def isFileEmpty(input_path):
    if(getSize(input_path)==0):return True
    else:return False
    

def folderTraversal(start_path):
    file_list_path=[]
    for dirpath, dirnames, filenames in os.walk(start_path):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            fp=fp.replace("\\","/")
            file_list_path.append(fp)
    return file_list_path



# WebPage Handlings

def writeWebPage(fname,html):
    with io.open(fname, "a", encoding="utf-8") as f:
        f.write(html)

def readWebPage(file_name):
    with open(file_name, "r", encoding="utf-8") as f:
        text = f.read()
        f.close()
    return text


from setuptools import setup

setup(name='utilum',
      version='0.1.2',
      description='Basic Utilities Package',
      packages = ['utilum', 'utilum/lookup'],
      install_requires = [],
      zip_safe = False,
      )


import random
import datetime

def spacingEndline(num):
    space=""
    endline="\n"
    for _ in range(num):
        space+=endline
    return space

def spacingSpace(num):
    space=""
    endline=" "
    for _ in range(num):
        space+=endline
    return space

def fullCapitalize(content):
    return content.upper()

def joinStringFromList(listString):
    temp=""
    for ils,ls in enumerate(listString):
        temp+=ls
        if(ils!=len(listString)):
            temp+="\n"
    
    return temp

def ampEncoder(input_string):
    input_string=input_string.replace("&Delta;","AMP_DELTA")
    input_string=input_string.replace("&diamondsuit;","AMP_DIAMONDSUIT")
    input_string=input_string.replace("&clubs;","AMP_CLUBS")
    input_string=input_string.replace("&nbsp;","AMP_NBSP")
    return input_string

def ampDecoder(input_string):
    input_string=input_string.replace("AMP_DELTA","&Delta;")
    input_string=input_string.replace("AMP_DIAMONDSUIT","&diamondsuit;")
    input_string=input_string.replace("AMP_CLUBS","&clubs;")
    input_string=input_string.replace("AMP_NBSP","&nbsp;")
    return input_string

def stringMatchFromList(input_string, input_list):
    for il in input_list:
        if(il in input_string):
            return True
        else:
            continue

    return False


def removeSpecialCharacters(input_string):
    special_character_string='''~`!@#$%^&*()_-+={[}]|\:;"'<,>.?/'''
    special_character_array=[c for c in special_character_string]

    output_string=input_string
    for sca in special_character_array:
        output_string=output_string.replace(sca,"")

    return output_string




def stringToUpperCase(input_string):
    return input_string.upper()



def stringToLowerCase(input_string):
    return input_string.lower()



def extraStringSpaceStrip(input_string):
    output_string=" ".join(input_string.split())
    return output_string





def stringToSet(input_string):
    output_string=input_string.split()
    return set(output_string)



def stringToList(input_string):
    input_string=input_string.split()
    return input_string



def listToString(input_list):
    return " ".join(input_list)










def extraSpaceStrip(input_string):
    output_string=input_string
    for _ in range(len(input_string)):
        output_string=output_string.replace("  "," ")
    return output_string





def stringPreprocess1(input_string):
    output_string=input_string
    output_string=extraSpaceStrip(output_string)
    output_string=stringToLowerCase(output_string)
    output_string=removeSpecialCharacters(output_string)
    return output_string


def setMatchScore1(input_set1, input_set2):
    output_set=input_set1.intersection(input_set2)
    return len(output_set),output_set


def randomCharStream(moreLen=4, withTimeStamp = True):
    charArray = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
    stream = ""
    for _ in range(moreLen):stream+=str(charArray[random.randint(0,len(charArray)-1)] )
    
    if(withTimeStamp == True):
        ddnsc = str(datetime.datetime.now()).replace("-", "_").replace(" ", "_").replace(":","_").replace(".","_")
        return stream + "_" + ddnsc
    else:
        return stream


def randomNumberStream(tokenLen=4):
    stream = ""
    for _ in range(tokenLen):stream+=str(random.randint(0,9))
    return stream


# Form Validator

class ValidatorLevel1Django:
    def __init__(self, request):
        self.request = request
        self.attrEntity = self.request.POST
        self.absentList = []
        self.emptyList = []
        self.inValidList = []

        self.absentDict = {}
        self.emptyDict = {}

    def addAttr(self, level1AttrName, allowEmpty = False):
        attrEntity = self.attrEntity

        combinedName = f"{level1AttrName}"
        # print("combinedName1: ", combinedName,level1AttrName, attrEntity,level1AttrName in self.attrEntity)
        if (level1AttrName in self.attrEntity):
            # print("combinedName2: ", combinedName)
            self.absentDict[combinedName] = False
            trimmedValue = str(attrEntity[level1AttrName]).replace(" ", "")
            if(allowEmpty == False):
                if(len(trimmedValue)==0):
                    self.emptyDict[combinedName] = True
                    self.emptyList.append({
                combinedName: {
                    "message": "Value Empty",
                    "param": combinedName,
                    "location": "POST",
                }})
                else:
                    self.emptyDict[combinedName] = False
            if(allowEmpty == True):
                if(len(trimmedValue)==0):
                    self.emptyDict[combinedName] = True
                else:
                    # print("combinedName: ", combinedName)
                    self.emptyDict[combinedName] = False
        else:
            self.absentDict[combinedName] = True
            self.absentList.append({
                combinedName: {
                    "message": "Attribute Absent",
                    "param": combinedName,
                    "location": "POST",
                }})

    def addRule(self, level1AttrName, ruleFunction):
        ruleEntity = self.request.POST[level1AttrName]

        [ ruleStatus, suggestionMessage ] = ruleFunction(ruleEntity)

        combinedName = f"{level1AttrName}"
        if (ruleStatus == True):pass
        else:
            self.inValidList.append({
                combinedName: {
                    "message": 'Invalid Value',
                    "suggestion": suggestionMessage,
                    "param": combinedName,
                    "location": "POST",
                }})

    def isAbsent(self):
        if(len(self.absentList)>0):return True
        else:return False
    def isEmpty(self):
        if(len(self.emptyList)>0):return True
        else:return False
    def isInvalid(self):
        if(len(self.inValidList)>0):return True
        else:return False

    def isValid(self):
        if(self.isAbsent()):return self.absentList
        elif(self.isEmpty()):return self.emptyList
        elif(self.isInvalid()):return self.inValidList
        else:return True
        

class ValidatorLevel1:
    def __init__(self, request, location):
        self.request = request
        self.location = location
        self.absentList = []
        self.emptyList = []
        self.inValidList = []

    def addAttr(self, level1AttrName, allowEmpty = False):
        if(self.location in self.request):
            attrEntity = self.request[self.location]
        else:
            attrEntity = None

        self.attrEntity = attrEntity
        combinedName = f"{level1AttrName}"
        if (level1AttrName in self.attrEntity):
            if(allowEmpty == False):
                trimmedValue = str(attrEntity[level1AttrName]).replace(" ", "")
                if(len(trimmedValue)==0):
                    self.emptyList.append({
                combinedName: {
                    "message": "Value Empty",
                    "param": combinedName,
                    "location": self.location,
                }})
        else:
            self.absentList.append({
                combinedName: {
                    "message": "Attribute Absent",
                    "param": combinedName,
                    "location": self.location,
                }})

    def addRule(self, level1AttrName, ruleFunction):
        if(self.location in self.request):
            ruleEntity = self.request[self.location][level1AttrName]
        else:
            ruleEntity = None

        [ ruleStatus, suggestionMessage ] = ruleFunction(ruleEntity)

        combinedName = f"{level1AttrName}"
        if (ruleStatus == True):pass
        else:
            self.inValidList.append({
                combinedName: {
                    "message": 'Invalid Value',
                    "suggestion": suggestionMessage,
                    "param": combinedName,
                    "location": self.location,
                }})

    def isAbsent(self):
        if(len(self.absentList)>0):return True
        else:return False
    def isEmpty(self):
        if(len(self.emptyList)>0):return True
        else:return False
    def isInvalid(self):
        if(len(self.inValidList)>0):return True
        else:return False


class ValidatorLevel2:
    def __init__(self, request, location):
        self.request = request
        self.location = location
        self.absentList = []
        self.emptyList = []
        self.inValidList = []

    def addAttr(self, level1AttrName, level2AttrName, allowEmpty = False):
        if(self.location in self.request):
            attrEntity = self.request[self.location][level1AttrName]
        else:
            attrEntity = None

        self.attrEntity = attrEntity
        combinedName = f"{level1AttrName}/{level2AttrName}"
        if (level2AttrName in self.attrEntity):
            if(allowEmpty == False):
                trimmedValue = str(attrEntity[level2AttrName]).replace(" ", "")
                if(len(trimmedValue)==0):
                    self.emptyList.append({
                combinedName: {
                    "message": "Value Empty",
                    "param": combinedName,
                    "location": self.location,
                }})
        else:
            self.absentList.append({
                combinedName: {
                    "message": "Attribute Absent",
                    "param": combinedName,
                    "location": self.location,
                }})        

    def addRule(self, level1AttrName, level2AttrName, ruleFunction):
        if(self.location in self.request):
            ruleEntity = self.request[self.location][level1AttrName][level2AttrName]
        else:
            ruleEntity = None

        [ ruleStatus, suggestionMessage ] = ruleFunction(ruleEntity)

        combinedName = f"{level1AttrName}/{level2AttrName}"
        if (ruleStatus == True):pass
        else:
            self.inValidList.append({
                combinedName: {
                    "message": 'Invalid Value',
                    "suggestion": suggestionMessage,
                    "param": combinedName,
                    "location": self.location,
                }})

    def isAbsent(self):
        if(len(self.absentList)>0):return True
        else:return False
    def isEmpty(self):
        if(len(self.emptyList)>0):return True
        else:return False
    def isInvalid(self):
        if(len(self.inValidList)>0):return True
        else:return False


def merge(current_array, new_array):
    for na in new_array:
        current_array.append(na)
    return current_array


def subtract(element, array):
    while element in array: array.remove(element)
    return array